@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2002 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2007 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,22 +20,26 @@
 #ifndef __PREFS_FILTER_H__
 #define __PREFS_FILTER_H__
 
-typedef enum
-{
-	FILTER_BY_NONE,
-	FILTER_BY_AUTO,
-	FILTER_BY_FROM,
-	FILTER_BY_TO,
-	FILTER_BY_SUBJECT
-} PrefsFilterType;
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#endif
 
-void prefs_filter_read_config	(void);
-void prefs_filter_write_config	(void);
-void prefs_filter_open		(const gchar	*header,
-				 const gchar	*key);
+#include <glib.h>
 
-void prefs_filter_rename_path	(const gchar	*old_path,
-				 const gchar	*new_path);
-void prefs_filter_delete_path	(const gchar	*path);
+#include "procmsg.h"
+
+void prefs_filter_open		(MsgInfo	*msginfo,
+				 const gchar	*header);
+
+GSList *prefs_filter_get_header_list		(void);
+GSList *prefs_filter_get_user_header_list	(void);
+
+gchar *prefs_filter_get_msg_header_field	(const gchar	*header_name);
+
+void prefs_filter_set_header_list		(MsgInfo	*msginfo);
+void prefs_filter_set_user_header_list		(GSList		*list);
+void prefs_filter_set_msg_header_list		(MsgInfo	*msginfo);
+
+void prefs_filter_write_user_header_list	(void);
 
 #endif /* __PREFS_FILTER_H__ */
